#include <kos.h>
#include <png/png.h>
#include "png.h"
#include "readpng.h"

/* load an n x n texture from a png */

#define LOAD565(r, g, b) (((r>>3)<<11) | ((g>>2)<<5) | ((b>>3)))
#define LOAD1555(r, g, b, a) (((a>>7)<<15)|((r>>3)<<10)|((g>>3)<<5)|((b>>3)))
#define LOAD4444(r, g, b, a) (((a>>4)<<12)|((r>>4)<<8)|((g>>4)<<4)|((b>>4)))

int png_to_texture(const char * filename, uint32 tex, uint32 mask)
{
    int i, j;
    unsigned short *ourbuffer;
    unsigned short *temp_tex;

    /* More stuff */
    FILE *infile;                /* source file */
    uch *buffer, *pRow;            /* Output row buffer */
    int row_stride;   /* physical row width in output buffer */
    int channels;           /* 3 for RGB 4 for RGBA */
    ulg width, height, n;
    /* 1.0 is normal for a PC Monitor
     * I have no clue for a Television
     */
    const double display_exponent = 1.0; 

    //    readpng_version_info();
    
    if ((infile = fopen(filename, "r")) == 0) {
      printf("png_to_texture: can't open %s\n", filename);
      return -1;
    }

    /* Step 1: Initialize loader */

    if (readpng_init(infile, &width, &height))
      return -1;
    n = MIN(width,height);

    /* Step 2: Read file */

    buffer = readpng_get_image(display_exponent, &channels, &row_stride);
    temp_tex = (unsigned short *)malloc(sizeof(unsigned short)*n*n);
    
    for(i = 0; i < n; i++)
    {
      pRow = &buffer[i*row_stride];
      ourbuffer = &temp_tex[i*n];

      if (channels == 3)
      {
        if (mask == PNG_NO_ALPHA)
          for(j = 0; j < n; j++)
            ourbuffer[j] = LOAD565(pRow[j*3], pRow[j*3+1], pRow[j*3+2]);
        else if (mask == PNG_MASK_ALPHA)
          for(j = 0; j < n; j++)
            ourbuffer[j] = LOAD1555(pRow[j*3],pRow[j*3+1],pRow[j*3+2],1);
        else if (mask == PNG_FULL_ALPHA)
          for(j = 0; j < n; j++)
            ourbuffer[j] = LOAD4444(pRow[j*3],pRow[j*3+1],pRow[j*3+2],1);
          
      }
      else if (channels == 4)
      {
        if (mask == PNG_NO_ALPHA)
        {
          for(j = 0; j < n; j++)
            ourbuffer[j] = LOAD565(pRow[j*4], pRow[j*4+1], pRow[j*4+2]);
        }
        else if (mask == PNG_MASK_ALPHA)
          for(j = 0; j < n; j++)
            ourbuffer[j] = LOAD1555(pRow[j*4],pRow[j*4+1],pRow[j*4+2],pRow[j*4+3]);
        else if (mask == PNG_FULL_ALPHA)
          for(j = 0; j < n; j++)
            ourbuffer[j] = LOAD4444(pRow[j*4],pRow[j*4+1],pRow[j*4+2],pRow[j*4+3]);
      }
    }
    txr_twiddle_copy(temp_tex, n, n, tex, n, n, 0);

    /* Step 3: Finish decompression */
    readpng_cleanup(1);

    fclose(infile);
    free(temp_tex);
    /* And we're done! */
    return 0;
}
